@@ -12,9 +12,9 @@ This module contains classes for XML Schema simple data types.
 """
 from decimal import DecimalException
 from typing import cast, Any, Callable, Dict, Iterator, List, \
-    Optional, Set, Union, Tuple, Type
+    Optional, Pattern, Set, Union, Tuple, Type
+from xml.etree import ElementTree
 
-from ..etree import etree_element
 from ..aliases import ElementType, AtomicValueType, ComponentClassType, \
     IterDecodeType, IterEncodeType, BaseXsdType, SchemaType, DecodedValueType, \
     EncodedValueType
@@ -26,6 +26,7 @@ from ..names import XSD_NAMESPACE, XSD_ANY_TYPE, XSD_SIMPLE_TYPE, XSD_PATTERN, \
     XSD_LIST, XSD_ANY_SIMPLE_TYPE, XSD_UNION, XSD_RESTRICTION, XSD_ANNOTATION, \
     XSD_ASSERTION, XSD_ID, XSD_IDREF, XSD_FRACTION_DIGITS, XSD_TOTAL_DIGITS, \
     XSD_EXPLICIT_TIMEZONE, XSD_ERROR, XSD_ASSERT, XSD_QNAME, XSD_UNTYPED_ATOMIC
+from ..translation import gettext as _
 from ..helpers import get_prefixed_qname, local_name
 
 from .exceptions import XMLSchemaValidationError, XMLSchemaEncodeError, \
@@ -95,6 +96,7 @@ class XsdSimpleType(XsdType, ValidationMixin[Union[str, bytes], DecodedValueType
             if white_space is not None:
                 self.white_space = white_space
 
+            p: Pattern[str]
             patterns = self.get_facet(XSD_PATTERN)
             if isinstance(patterns, XsdPatternFacets):
                 self.patterns = patterns
@@ -129,24 +131,24 @@ class XsdSimpleType(XsdType, ValidationMixin[Union[str, bytes], DecodedValueType
         if facets and self.base_type is not None:
             if isinstance(self.base_type, XsdSimpleType):
                 if self.base_type.name == XSD_ANY_SIMPLE_TYPE:
-                    self.parse_error(
-                        "facets not allowed for a direct derivation of xs:anySimpleType"
-                    )
+                    msg = _("facets not allowed for a direct derivation of xs:anySimpleType")
+                    self.parse_error(msg)
             elif self.base_type.has_simple_content():
                 if self.base_type.content.name == XSD_ANY_SIMPLE_TYPE:
-                    self.parse_error(
-                        "facets not allowed for a direct content derivation of xs:anySimpleType"
-                    )
+                    msg = _("facets not allowed for a direct content "
+                            "derivation of xs:anySimpleType")
+                    self.parse_error(msg)
 
         # Checks the applicability of the facets
         if any(k not in self.admitted_facets for k in facets if k is not None):
-            reason = "one or more facets are not applicable, admitted set is %r:"
-            self.parse_error(reason % {local_name(e) for e in self.admitted_facets if e})
+            msg = _("one or more facets are not applicable, admitted set is {!r}")
+            self.parse_error(msg.format({local_name(e) for e in self.admitted_facets if e}))
 
         # Check group base_type
         base_type = {t.base_type for t in facets.values() if isinstance(t, XsdFacet)}
         if len(base_type) > 1:
-            self.parse_error("facet group must have the same base_type: %r" % base_type)
+            msg = _("facet group must have the same base type: %r")
+            self.parse_error(msg % base_type)
         base_type = base_type.pop() if base_type else None
 
         # Checks length based facets
@@ -155,46 +157,61 @@ class XsdSimpleType(XsdType, ValidationMixin[Union[str, bytes], DecodedValueType
         max_length = getattr(facets.get(XSD_MAX_LENGTH), 'value', None)
         if length is not None:
             if length < 0:
-                self.parse_error("'length' value must be non negative integer")
+                self.parse_error(_("'length' value must be non a negative integer"))
+
             if min_length is not None:
                 if min_length > length:
-                    self.parse_error("'minLength' value must be less or equal to 'length'")
+                    msg = _("'minLength' value must be less than or equal to 'length'")
+                    self.parse_error(msg)
                 min_length_facet = base_type.get_facet(XSD_MIN_LENGTH)
                 length_facet = base_type.get_facet(XSD_LENGTH)
                 if (min_length_facet is None
                         or (length_facet is not None
                             and length_facet.base_type == min_length_facet.base_type)):
-                    self.parse_error("cannot specify both 'length' and 'minLength'")
+                    msg = _("cannot specify both 'length' and 'minLength'")
+                    self.parse_error(msg)
+
             if max_length is not None:
                 if max_length < length:
-                    self.parse_error("'maxLength' value must be greater or equal to 'length'")
+                    msg = _("'maxLength' value must be greater or equal to 'length'")
+                    self.parse_error(msg)
+
                 max_length_facet = base_type.get_facet(XSD_MAX_LENGTH)
                 length_facet = base_type.get_facet(XSD_LENGTH)
                 if max_length_facet is None \
                         or (length_facet is not None
                             and length_facet.base_type == max_length_facet.base_type):
-                    self.parse_error("cannot specify both 'length' and 'maxLength'")
+                    msg = _("cannot specify both 'length' and 'maxLength'")
+                    self.parse_error(msg)
+
             min_length = max_length = length
         elif min_length is not None or max_length is not None:
             min_length_facet = base_type.get_facet(XSD_MIN_LENGTH)
             max_length_facet = base_type.get_facet(XSD_MAX_LENGTH)
             if min_length is not None:
                 if min_length < 0:
-                    self.parse_error("'minLength' value must be non negative integer")
+                    msg = _("'minLength' value must be a non negative integer")
+                    self.parse_error(msg)
                 if max_length is not None and max_length < min_length:
-                    self.parse_error("'maxLength' value is lesser than 'minLength'")
+                    msg = _("'maxLength' value is less than 'minLength'")
+                    self.parse_error(msg)
                 if min_length_facet is not None and min_length_facet.value > min_length:
-                    self.parse_error("'minLength' has a lesser value than parent")
+                    msg = _("'minLength' has a lesser value than parent")
+                    self.parse_error(msg)
                 if max_length_facet is not None and min_length > max_length_facet.value:
-                    self.parse_error("'minLength' has a greater value than parent 'maxLength'")
+                    msg = _("'minLength' has a greater value than parent 'maxLength'")
+                    self.parse_error(msg)
 
             if max_length is not None:
                 if max_length < 0:
-                    self.parse_error("'maxLength' value mu  st be non negative integer")
+                    msg = _("'maxLength' value must be a non negative integer")
+                    self.parse_error(msg)
                 if min_length_facet is not None and min_length_facet.value > max_length:
-                    self.parse_error("'maxLength' has a lesser value than parent 'minLength'")
+                    msg = _("'maxLength' has a lesser value than parent 'minLength'")
+                    self.parse_error(msg)
                 if max_length_facet is not None and max_length > max_length_facet.value:
-                    self.parse_error("'maxLength' has a greater value than parent")
+                    msg = _("'maxLength' has a greater value than parent")
+                    self.parse_error(msg)
 
         # Checks min/max values
         min_inclusive = getattr(facets.get(XSD_MIN_INCLUSIVE), 'value', None)
@@ -204,39 +221,48 @@ class XsdSimpleType(XsdType, ValidationMixin[Union[str, bytes], DecodedValueType
 
         if min_inclusive is not None:
             if min_exclusive is not None:
-                self.parse_error("cannot specify both 'minInclusive' and 'minExclusive")
+                msg = _("cannot specify both 'minInclusive' and 'minExclusive'")
+                self.parse_error(msg)
             if max_inclusive is not None and min_inclusive > max_inclusive:
-                self.parse_error("'minInclusive' must be less or equal to 'maxInclusive'")
+                msg = _("'minInclusive' must be less or equal to 'maxInclusive'")
+                self.parse_error(msg)
             elif max_exclusive is not None and min_inclusive >= max_exclusive:
-                self.parse_error("'minInclusive' must be lesser than 'maxExclusive'")
+                msg = _("'minInclusive' must be lesser than 'maxExclusive'")
+                self.parse_error(msg)
 
         elif min_exclusive is not None:
             if max_inclusive is not None and min_exclusive >= max_inclusive:
-                self.parse_error("'minExclusive' must be lesser than 'maxInclusive'")
+                msg = _("'minExclusive' must be lesser than 'maxInclusive'")
+                self.parse_error(msg)
             elif max_exclusive is not None and min_exclusive > max_exclusive:
-                self.parse_error("'minExclusive' must be less or equal to 'maxExclusive'")
+                msg = _("'minExclusive' must be less or equal to 'maxExclusive'")
+                self.parse_error(msg)
 
         if max_inclusive is not None and max_exclusive is not None:
-            self.parse_error("cannot specify both 'maxInclusive' and 'maxExclusive")
+            self.parse_error(_("cannot specify both 'maxInclusive' and 'maxExclusive'"))
 
         # Checks fraction digits
         if XSD_TOTAL_DIGITS in facets:
             if XSD_FRACTION_DIGITS in facets and \
                     facets[XSD_TOTAL_DIGITS].value < facets[XSD_FRACTION_DIGITS].value:
-                self.parse_error("fractionDigits facet value cannot be lesser than the "
-                                 "value of totalDigits facet")
+                msg = _("fractionDigits facet value cannot be lesser "
+                        "than the value of totalDigits facet")
+                self.parse_error(msg)
+
             total_digits = base_type.get_facet(XSD_TOTAL_DIGITS)
             if total_digits is not None and total_digits.value < facets[XSD_TOTAL_DIGITS].value:
-                self.parse_error("totalDigits facet value cannot be greater than "
-                                 "the value of the same facet in the base type")
+                msg = _("totalDigits facet value cannot be greater than "
+                        "the value of the same facet in the base type")
+                self.parse_error(msg)
 
         # Checks XSD 1.1 facets
         if XSD_EXPLICIT_TIMEZONE in facets:
             explicit_tz_facet = base_type.get_facet(XSD_EXPLICIT_TIMEZONE)
             if explicit_tz_facet and explicit_tz_facet.value in ('prohibited', 'required') \
                     and facets[XSD_EXPLICIT_TIMEZONE].value != explicit_tz_facet.value:
-                self.parse_error("the explicitTimezone facet value cannot be changed if the base "
-                                 "type has the same facet with value %r" % explicit_tz_facet.value)
+                msg = _("the explicitTimezone facet value cannot be changed "
+                        "if the base type has the same facet with value %r")
+                self.parse_error(msg % explicit_tz_facet.value)
 
         self.min_length = min_length
         self.max_length = max_length
@@ -253,8 +279,14 @@ class XsdSimpleType(XsdType, ValidationMixin[Union[str, bytes], DecodedValueType
     def min_value(self) -> Optional[AtomicValueType]:
         min_exclusive: Optional['AtomicValueType']
         min_inclusive: Optional['AtomicValueType']
-        min_exclusive = getattr(self.get_facet(XSD_MIN_EXCLUSIVE), 'value', None)
-        min_inclusive = getattr(self.get_facet(XSD_MIN_INCLUSIVE), 'value', None)
+        min_exclusive = cast(
+            Optional['AtomicValueType'],
+            getattr(self.get_facet(XSD_MIN_EXCLUSIVE), 'value', None)
+        )
+        min_inclusive = cast(
+            Optional['AtomicValueType'],
+            getattr(self.get_facet(XSD_MIN_INCLUSIVE), 'value', None)
+        )
 
         if min_exclusive is None:
             return min_inclusive
@@ -269,8 +301,14 @@ class XsdSimpleType(XsdType, ValidationMixin[Union[str, bytes], DecodedValueType
     def max_value(self) -> Optional[AtomicValueType]:
         max_exclusive: Optional['AtomicValueType']
         max_inclusive: Optional['AtomicValueType']
-        max_exclusive = getattr(self.get_facet(XSD_MAX_EXCLUSIVE), 'value', None)
-        max_inclusive = getattr(self.get_facet(XSD_MAX_INCLUSIVE), 'value', None)
+        max_exclusive = cast(
+            Optional['AtomicValueType'],
+            getattr(self.get_facet(XSD_MAX_EXCLUSIVE), 'value', None)
+        )
+        max_inclusive = cast(
+            Optional['AtomicValueType'],
+            getattr(self.get_facet(XSD_MAX_INCLUSIVE), 'value', None)
+        )
 
         if max_exclusive is None:
             return max_inclusive
@@ -420,7 +458,7 @@ class XsdSimpleType(XsdType, ValidationMixin[Union[str, bytes], DecodedValueType
     def iter_encode(self, obj: Any, validation: str = 'lax', **kwargs: Any) \
             -> IterEncodeType[EncodedValueType]:
         if not isinstance(obj, (str, bytes)):
-            reason = "a {!r} or {!r} object required".format(str, bytes)
+            reason = _("a {0!r} or {1!r} object required").format(str, bytes)
             yield XMLSchemaEncodeError(self, obj, str, reason)
         else:
             text = self.normalize(obj)
@@ -575,7 +613,7 @@ class XsdAtomicBuiltin(XsdAtomic):
         if isinstance(obj, (str, bytes)):
             obj = self.normalize(obj)
         elif obj is not None and not isinstance(obj, self.instance_types):
-            reason = "value is not an instance of {!r}".format(self.instance_types)
+            reason = _("value is not an instance of {!r}").format(self.instance_types)
             yield XMLSchemaDecodeError(self, obj, self.to_python, reason)
 
         if validation == 'skip':
@@ -598,8 +636,8 @@ class XsdAtomicBuiltin(XsdAtomic):
             yield None
             return
         except TypeError:
-            # xs:error type (eg. an XSD 1.1 type alternative used to catch invalid values)
-            reason = "invalid value {!r}".format(obj)
+            # xs:error type (e.g. an XSD 1.1 type alternative used to catch invalid values)
+            reason = _("invalid value {!r}").format(obj)
             yield self.validation_error(validation, error=reason, obj=obj)
             yield None
             return
@@ -620,11 +658,11 @@ class XsdAtomicBuiltin(XsdAtomic):
                     pass
                 else:
                     try:
-                        result = '{%s}%s' % (kwargs['namespaces'][prefix], name)
+                        result = f"{{{kwargs['namespaces'][prefix]}}}{name}"
                     except (TypeError, KeyError):
                         try:
                             if kwargs['source'].namespace != XSD_NAMESPACE:
-                                reason = "unmapped prefix %r on QName" % prefix
+                                reason = _("unmapped prefix %r in a QName") % prefix
                                 yield self.validation_error(validation, error=reason, obj=obj)
                         except KeyError:
                             pass
@@ -635,7 +673,7 @@ class XsdAtomicBuiltin(XsdAtomic):
                     pass
                 else:
                     if default_namespace:
-                        result = '{%s}%s' % (default_namespace, obj)
+                        result = f'{{{default_namespace}}}{obj}'
 
         elif self.name == XSD_IDREF:
             try:
@@ -658,19 +696,19 @@ class XsdAtomicBuiltin(XsdAtomic):
                     if not id_map[obj]:
                         id_map[obj] = 1
                     else:
-                        reason = "duplicated xs:ID value {!r}".format(obj)
+                        reason = _("duplicated xs:ID value {!r}").format(obj)
                         yield self.validation_error(validation, error=reason, obj=obj)
                 else:
                     if not id_map[obj]:
                         id_map[obj] = 1
                         id_list.append(obj)
                         if len(id_list) > 1 and self.xsd_version == '1.0':
-                            reason = "no more than one attribute of type ID should " \
-                                     "be present in an element"
+                            reason = _("no more than one attribute of type ID should "
+                                       "be present in an element")
                             yield self.validation_error(validation, reason, obj, **kwargs)
 
                     elif obj not in id_list or self.xsd_version == '1.0':
-                        reason = "duplicated xs:ID value {!r}".format(obj)
+                        reason = _("duplicated xs:ID value {!r}").format(obj)
                         yield self.validation_error(validation, error=reason, obj=obj)
 
         yield result
@@ -690,12 +728,12 @@ class XsdAtomicBuiltin(XsdAtomic):
         elif isinstance(obj, bool):
             types_: Any = self.instance_types
             if types_ is not bool or (isinstance(types_, tuple) and bool in types_):
-                reason = "boolean value {!r} requires a {!r} decoder".format(obj, bool)
+                reason = _("boolean value {0!r} requires a {1!r} decoder").format(obj, bool)
                 yield XMLSchemaEncodeError(self, obj, self.from_python, reason)
                 obj = self.python_type(obj)
 
         elif not isinstance(obj, self.instance_types):
-            reason = "{!r} is not an instance of {!r}".format(obj, self.instance_types)
+            reason = _("{0!r} is not an instance of {1!r}").format(obj, self.instance_types)
             yield XMLSchemaEncodeError(self, obj, self.from_python, reason)
 
             try:
@@ -712,7 +750,7 @@ class XsdAtomicBuiltin(XsdAtomic):
                 if value == obj or str(value) == str(obj):
                     obj = value
                 else:
-                    reason = "Invalid value {!r}".format(obj)
+                    reason = _("invalid value {!r}").format(obj)
                     yield XMLSchemaEncodeError(self, obj, self.from_python, reason)
                     yield None
                     return
@@ -751,7 +789,7 @@ class XsdList(XsdSimpleType):
     """
     base_type: XsdSimpleType
     _ADMITTED_TAGS = {XSD_LIST}
-    _white_space_elem = etree_element(
+    _white_space_elem = ElementTree.Element(
         XSD_WHITE_SPACE, attrib={'value': 'collapse', 'fixed': 'true'}
     )
 
@@ -777,10 +815,14 @@ class XsdList(XsdSimpleType):
                     if child.tag == XSD_LIST:
                         super(XsdList, self).__setattr__(name, child)
                         return
-            raise XMLSchemaValueError("a %r definition required for %r." % (XSD_LIST, self))
+            raise XMLSchemaValueError(
+                "a {0!r} definition required for {1!r}".format(XSD_LIST, self)
+            )
         elif name == 'base_type':
             if not value.is_atomic():
-                raise XMLSchemaValueError("%r: a list must be based on atomic data types." % self)
+                raise XMLSchemaValueError(
+                    _("%r: a list must be based on atomic data types") % self
+                )
         elif name == 'white_space' and value is None:
             value = 'collapse'
         super(XsdList, self).__setattr__(name, value)
@@ -798,7 +840,7 @@ class XsdList(XsdSimpleType):
                 base_type = self.any_atomic_type
 
             if 'itemType' in self.elem.attrib:
-                self.parse_error("ambiguous list type declaration")
+                self.parse_error(_("ambiguous list type declaration"))
 
         else:
             # List tag with itemType attribute that refers to a global type
@@ -806,7 +848,7 @@ class XsdList(XsdSimpleType):
                 item_qname = self.schema.resolve_qname(self.elem.attrib['itemType'])
             except (KeyError, ValueError, RuntimeError) as err:
                 if 'itemType' not in self.elem.attrib:
-                    self.parse_error("missing list type declaration")
+                    self.parse_error(_("missing list type declaration"))
                 else:
                     self.parse_error(err)
                 base_type = self.any_atomic_type
@@ -814,22 +856,22 @@ class XsdList(XsdSimpleType):
                 try:
                     base_type = self.maps.lookup_type(item_qname)
                 except KeyError:
-                    self.parse_error("unknown itemType %r" % self.elem.attrib['itemType'])
+                    msg = _("unknown type {!r}")
+                    self.parse_error(msg.format(self.elem.attrib['itemType']))
                     base_type = self.any_atomic_type
                 else:
                     if isinstance(base_type, tuple):
-                        self.parse_error(
-                            "circular definition found for type {!r}".format(item_qname)
-                        )
+                        msg = _("circular definition found for type {!r}")
+                        self.parse_error(msg.format(item_qname))
                         base_type = self.any_atomic_type
 
         if base_type.final == '#all' or 'list' in base_type.final:
-            self.parse_error(
-                "'final' value of the itemType %r forbids derivation by list" % base_type
-            )
+            msg = _("'final' value of the itemType %r forbids derivation by list")
+            self.parse_error(msg % base_type)
 
         if base_type.name == XSD_ANY_ATOMIC_TYPE:
-            self.parse_error("Cannot use xs:anyAtomicType as base type of a user-defined type")
+            msg = _("cannot use xs:anyAtomicType as base type of a user-defined type")
+            self.parse_error(msg)
 
         try:
             self.base_type = base_type
@@ -945,11 +987,14 @@ class XsdUnion(XsdSimpleType):
                     if child.tag == XSD_UNION:
                         super(XsdUnion, self).__setattr__(name, child)
                         return
-            raise XMLSchemaValueError("a %r definition required for %r." % (XSD_UNION, self))
+            raise XMLSchemaValueError(
+                "a {0!r} definition required for {1!r}".format(XSD_UNION, self)
+            )
 
         elif name == 'white_space':
             if not (value is None or value == 'collapse'):
-                raise XMLSchemaValueError("Wrong value % for attribute 'white_space'." % value)
+                msg = _("wrong value %r for attribute 'white_space'")
+                raise XMLSchemaValueError(msg % value)
             value = 'collapse'
         super(XsdUnion, self).__setattr__(name, value)
 
@@ -976,31 +1021,32 @@ class XsdUnion(XsdSimpleType):
                 try:
                     mt = self.maps.lookup_type(type_qname)
                 except KeyError:
-                    self.parse_error("unknown member type %r" % type_qname)
+                    self.parse_error(_("unknown type {!r}").format(type_qname))
                     mt = self.any_atomic_type
                 except XMLSchemaParseError as err:
                     self.parse_error(err)
                     mt = self.any_atomic_type
 
                 if isinstance(mt, tuple):
-                    self.parse_error(
-                        "circular definition found on xs:union type {!r}".format(self.name)
-                    )
+                    msg = _("circular definition found on xs:union type {!r}")
+                    self.parse_error(msg.format(self.name))
                     continue
                 elif not isinstance(mt, self._ADMITTED_TYPES):
-                    self.parse_error("a {!r} required, not {!r}".format(self._ADMITTED_TYPES, mt))
+                    msg = _("a {0!r} required, not {1!r}")
+                    self.parse_error(msg.format(self._ADMITTED_TYPES, mt))
                     continue
                 elif mt.final == '#all' or 'union' in mt.final:
-                    self.parse_error("'final' value of the memberTypes %r "
-                                     "forbids derivation by union" % member_types)
+                    msg = _("'final' value of the memberTypes %r forbids derivation by union")
+                    self.parse_error(msg % member_types)
 
                 member_types.append(mt)
 
         if not member_types:
-            self.parse_error("missing xs:union type declarations")
+            self.parse_error(_("missing xs:union type declarations"))
             self.member_types = [self.any_atomic_type]
         elif any(mt.name == XSD_ANY_ATOMIC_TYPE for mt in member_types):
-            self.parse_error("Cannot use xs:anyAtomicType as base type of a user-defined type")
+            msg = _("cannot use xs:anyAtomicType as base type of a user-defined type")
+            self.parse_error(msg)
         else:
             self.member_types = member_types
             if all(not mt.allow_empty for mt in member_types):
@@ -1058,7 +1104,7 @@ class XsdUnion(XsdSimpleType):
                 break
 
         if ' ' not in obj.strip():
-            reason = "invalid value %r." % obj
+            reason = _("invalid value {!r}").format(obj)
             yield XMLSchemaDecodeError(self, obj, self.member_types, reason)
 
         items = []
@@ -1079,7 +1125,7 @@ class XsdUnion(XsdSimpleType):
                     items.append(str(chunk))
 
         if not_decodable:
-            reason = "no type suitable for decoding the values %r." % not_decodable
+            reason = _("no type suitable for decoding the values %r") % not_decodable
             yield XMLSchemaDecodeError(self, obj, self.member_types, reason)
 
         yield items if len(items) > 1 else items[0] if items else None
@@ -1094,7 +1140,7 @@ class XsdUnion(XsdSimpleType):
                     return
                 elif validation == 'strict':
                     # In 'strict' mode avoid lax encoding by similar types
-                    # (eg. float encoded by int)
+                    # (e.g. float encoded by int)
                     break
 
         if hasattr(obj, '__iter__') and not isinstance(obj, (str, bytes)):
@@ -1113,7 +1159,7 @@ class XsdUnion(XsdSimpleType):
                     break
 
         if validation != 'skip':
-            reason = "no type suitable for encoding the object."
+            reason = _("no type suitable for encoding the object")
             yield XMLSchemaEncodeError(self, obj, self.member_types, reason)
             yield None
         else:
@@ -1161,7 +1207,8 @@ class XsdAtomicRestriction(XsdAtomic):
             elem = cast(ElementType, self._parse_child_component(elem))
 
         if self.name is not None and self.parent is not None:
-            self.parse_error("'name' attribute in a local simpleType definition")
+            msg = _("'name' attribute in a local simpleType definition")
+            self.parse_error(msg)
 
         base_type: Any = None
         facets: Any = {}
@@ -1177,46 +1224,49 @@ class XsdAtomicRestriction(XsdAtomic):
             else:
                 if base_qname == self.name:
                     if self.redefine is None:
-                        self.parse_error("wrong definition with self-reference")
+                        msg = _("wrong definition with self-reference")
+                        self.parse_error(msg)
                         base_type = self.any_atomic_type
                     else:
                         base_type = self.base_type
                 else:
                     if self.redefine is not None:
-                        self.parse_error("wrong redefinition without self-reference")
+                        msg = _("wrong redefinition without self-reference")
+                        self.parse_error(msg)
 
                     try:
                         base_type = self.maps.lookup_type(base_qname)
                     except KeyError:
-                        self.parse_error("unknown type %r" % elem.attrib['base'])
+                        self.parse_error(_("unknown type {!r}").format(elem.attrib['base']))
                         base_type = self.any_atomic_type
                     except XMLSchemaParseError as err:
                         self.parse_error(err)
                         base_type = self.any_atomic_type
                     else:
                         if isinstance(base_type, tuple):
-                            msg = "circularity definition between {!r} and {!r}"
+                            msg = _("circular definition found between {0!r} and {1!r}")
                             self.parse_error(msg.format(self, base_qname))
                             base_type = self.any_atomic_type
 
             if base_type.is_simple() and base_type.name == XSD_ANY_SIMPLE_TYPE:
-                self.parse_error("wrong base type {!r}, an atomic type required")
+                msg = _("wrong base type %r, an atomic type required")
+                self.parse_error(msg % XSD_ANY_SIMPLE_TYPE)
             elif base_type.is_complex():
                 if base_type.mixed and base_type.is_emptiable():
                     child = self._parse_child_component(elem, strict=False)
                     if child is None:
-                        self.parse_error("an xs:simpleType definition expected")
+                        msg = _("an xs:simpleType definition expected")
+                        self.parse_error(msg)
                     elif child.tag != XSD_SIMPLE_TYPE:
                         # See: "http://www.w3.org/TR/xmlschema-2/#element-restriction"
-                        self.parse_error(
+                        self.parse_error(_(
                             "when a complexType with simpleContent restricts a complexType "
                             "with mixed and with emptiable content then a simpleType child "
-                            "declaration is required."
-                        )
+                            "declaration is required"
+                        ))
                 elif self.parent is None or self.parent.is_simple():
-                    self.parse_error(
-                        "simpleType restriction of %r is not allowed" % base_type
-                    )
+                    msg = _("simpleType restriction of %r is not allowed")
+                    self.parse_error(msg % base_type)
 
         for child in elem:
             if child.tag == XSD_ANNOTATION or callable(child.tag):
@@ -1224,11 +1274,13 @@ class XsdAtomicRestriction(XsdAtomic):
             elif child.tag in self._CONTENT_TAIL_TAGS:
                 has_attributes = True  # only if it's a complexType restriction
             elif has_attributes:
-                self.parse_error("unexpected tag after attribute declarations")
+                msg = _("unexpected tag after attribute declarations")
+                self.parse_error(msg)
             elif child.tag == XSD_SIMPLE_TYPE:
                 # Case of simpleType declaration inside a restriction
                 if has_simple_type_child:
-                    self.parse_error("duplicated simpleType declaration")
+                    msg = _("duplicated simpleType declaration")
+                    self.parse_error(msg)
 
                 if base_type is None:
                     try:
@@ -1249,22 +1301,22 @@ class XsdAtomicRestriction(XsdAtomic):
                             final=base_type.final,
                         )
                 elif 'base' in elem.attrib:
-                    self.parse_error(
-                        "restriction with 'base' attribute and simpleType declaration"
-                    )
+                    msg = _("restriction with 'base' attribute and simpleType declaration")
+                    self.parse_error(msg)
 
                 has_simple_type_child = True
             else:
                 try:
                     facet_class = self._FACETS_BUILDERS[child.tag]
                 except KeyError:
-                    self.parse_error("unexpected tag %r in restriction:" % child.tag)
+                    self.parse_error(_("unexpected tag %r in restriction") % child.tag)
                     continue
 
                 if child.tag not in facets:
                     facets[child.tag] = facet_class(child, self.schema, self, base_type)
                 elif child.tag not in MULTIPLE_FACETS:
-                    self.parse_error("multiple %r constraint facet" % local_name(child.tag))
+                    msg = _("multiple %r constraint facet")
+                    self.parse_error(msg % local_name(child.tag))
                 elif child.tag != XSD_ASSERTION:
                     facets[child.tag].append(child)
                 else:
@@ -1275,12 +1327,13 @@ class XsdAtomicRestriction(XsdAtomic):
                         facets[child.tag] = [facets[child.tag], assertion]
 
         if base_type is None:
-            self.parse_error("missing base type in restriction:")
+            self.parse_error(_("missing base type in restriction"))
         elif base_type.final == '#all' or 'restriction' in base_type.final:
-            self.parse_error("'final' value of the baseType %r forbids "
-                             "derivation by restriction" % base_type)
+            msg = _("'final' value of the baseType %r forbids derivation by restriction")
+            self.parse_error(msg % base_type)
         if base_type is self.any_atomic_type:
-            self.parse_error("Cannot use xs:anyAtomicType as base type of a user-defined type")
+            msg = _("cannot use xs:anyAtomicType as base type of a user-defined type")
+            self.parse_error(msg)
 
         self.base_type = base_type
         self.facets = facets
@@ -1325,8 +1378,9 @@ class XsdAtomicRestriction(XsdAtomic):
             yield obj
             return
         else:
-            raise XMLSchemaValueError("wrong base type %r: a simpleType or a complexType with "
-                                      "simple or mixed content required." % self.base_type)
+            msg = _("wrong base type %r: a simpleType or a complexType "
+                    "with simple or mixed content required")
+            raise XMLSchemaValueError(msg % self.base_type)
 
         if self.patterns:
             if not isinstance(self.primitive_type, XsdUnion):
@@ -1340,8 +1394,6 @@ class XsdAtomicRestriction(XsdAtomic):
         for result in base_type.iter_decode(obj, validation, **kwargs):
             if isinstance(result, XMLSchemaValidationError):
                 yield result
-                if isinstance(result, XMLSchemaDecodeError):
-                    yield str(obj) if validation == 'skip' else None
             else:
                 if result is not None:
                     for validator in self.validators:
@@ -1373,8 +1425,9 @@ class XsdAtomicRestriction(XsdAtomic):
                 yield str(obj)
                 return
             else:
-                raise XMLSchemaValueError("wrong base type %r: a simpleType or a complexType with "
-                                          "simple or mixed content required." % self.base_type)
+                msg = _("wrong base type %r: a simpleType or a complexType "
+                        "with simple or mixed content required")
+                raise XMLSchemaValueError(msg % self.base_type)
 
         result: Any
         for result in base_type.iter_encode(obj, validation):
