@@ -13,6 +13,8 @@ This module contains classes for XML Schema wildcards.
 from typing import cast, Any, Callable, Dict, Iterable, Iterator, List, Optional, \
     Tuple, Union, Counter
 
+from elementpath import SchemaElementNode, build_schema_node_tree
+
 from ..exceptions import XMLSchemaValueError
 from ..names import XSI_NAMESPACE, XSD_ANY, XSD_ANY_ATTRIBUTE, \
     XSD_OPEN_CONTENT, XSD_DEFAULT_OPEN_CONTENT, XSI_TYPE
@@ -21,7 +23,7 @@ from ..aliases import ElementType, SchemaType, SchemaElementType, SchemaAttribut
     DecodedValueType, EncodedValueType
 from ..translation import gettext as _
 from ..helpers import get_namespace, raw_xml_encode
-from ..xpath import XMLSchemaProtocol, ElementProtocol, XMLSchemaProxy, ElementPathMixin
+from ..xpath import XsdSchemaProtocol, XsdElementProtocol, XMLSchemaProxy, ElementPathMixin
 from .xsdbase import ValidationMixin, XsdComponent
 from .particles import ParticleMixin
 from . import elements
@@ -150,6 +152,10 @@ class XsdWildcard(XsdComponent):
     def built(self) -> bool:
         return True
 
+    @property
+    def value_constraint(self) -> Optional[str]:
+        return None
+
     def is_matching(self, name: Optional[str],
                     default_namespace: Optional[str] = None,
                     **kwargs: Any) -> bool:
@@ -200,12 +206,14 @@ class XsdWildcard(XsdComponent):
 
     def is_restriction(self, other: Union[ModelParticleType, 'XsdAnyAttribute'],
                        check_occurs: bool = True) -> bool:
-        if check_occurs and isinstance(self, ParticleMixin) \
-                and not isinstance(other, XsdAnyAttribute) \
-                and not self.has_occurs_restriction(other):
-            return False
-        elif not isinstance(other, self.__class__):
+        if not isinstance(other, self.__class__):
             return False
+        elif check_occurs and isinstance(self, ParticleMixin):
+            if not isinstance(other, XsdAnyAttribute) and \
+                    not self.has_occurs_restriction(other):
+                return False
+            elif self.max_occurs == 0:
+                return True
 
         other: XsdWildcard  # type: ignore[no-redef]
         if other.process_contents == 'strict' and self.process_contents != 'strict':
@@ -419,8 +427,21 @@ class XsdAnyElement(XsdWildcard, ParticleMixin,
     @property
     def xpath_proxy(self) -> XMLSchemaProxy:
         return XMLSchemaProxy(
-            schema=cast(XMLSchemaProtocol, self.schema),
-            base_element=cast(ElementProtocol, self)
+            schema=cast(XsdSchemaProtocol, self.schema),
+            base_element=cast(XsdElementProtocol, self)
+        )
+
+    @property
+    def xpath_node(self) -> SchemaElementNode:
+        schema_node = self.schema.xpath_node
+        node = schema_node.get_element_node(cast(XsdElementProtocol, self))
+        if isinstance(node, SchemaElementNode):
+            return node
+
+        return build_schema_node_tree(
+            root=cast(XsdElementProtocol, self),
+            elements=schema_node.elements,
+            global_elements=schema_node.children,
         )
 
     def _parse(self) -> None:
@@ -447,7 +468,7 @@ class XsdAnyElement(XsdWildcard, ParticleMixin,
 
         try:
             if name[0] != '{' and default_namespace:
-                return self.maps.lookup_element('{%s}%s' % (default_namespace, name))
+                return self.maps.lookup_element(f'{{{default_namespace}}}{name}')
             else:
                 return self.maps.lookup_element(name)
         except LookupError:
@@ -530,7 +551,7 @@ class XsdAnyElement(XsdWildcard, ParticleMixin,
                 yield from xsd_element.iter_encode(value, validation, **kwargs)
                 return
 
-        # Check if there is an xsi:type attribute, but it has to extract
+        # Check if there is a xsi:type attribute, but it has to extract
         # attributes using the converter instance.
         if self.process_contents == 'strict':
             xsd_element = self.maps.validator.create_element(
@@ -641,7 +662,7 @@ class XsdAnyAttribute(XsdWildcard, ValidationMixin[Tuple[str, str], DecodedValue
 
         try:
             if name[0] != '{' and default_namespace:
-                return self.maps.lookup_attribute('{%s}%s' % (default_namespace, name))
+                return self.maps.lookup_attribute(f'{{{default_namespace}}}{name}')
             else:
                 return self.maps.lookup_attribute(name)
         except LookupError:
@@ -752,7 +773,7 @@ class Xsd11AnyElement(XsdAnyElement):
             if not self.is_namespace_allowed(''):
                 return False
         else:
-            name = '{%s}%s' % (default_namespace, name)
+            name = f'{{{default_namespace}}}{name}'
             if not self.is_namespace_allowed('') \
                     and not self.is_namespace_allowed(default_namespace):
                 return False
@@ -815,7 +836,7 @@ class Xsd11AnyAttribute(XsdAnyAttribute):
         elif not default_namespace:
             namespace = ''
         else:
-            name = '{%s}%s' % (default_namespace, name)
+            name = f'{{{default_namespace}}}{name}'
             namespace = default_namespace
 
         if '##defined' in self.not_qname and name in self.maps.attributes:
